@@ -1,37 +1,41 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
-// All rights reserved.
+// Copyright 2006-2012, Paul Beckingham, Federico Hernandez.
 //
-// This program is free software; you can redistribute it and/or modify it under
-// the terms of the GNU General Public License as published by the Free Software
-// Foundation; either version 2 of the License, or (at your option) any later
-// version.
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to deal
+// in the Software without restriction, including without limitation the rights
+// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+// copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
 //
-// This program is distributed in the hope that it will be useful, but WITHOUT
-// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
-// details.
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
 //
-// You should have received a copy of the GNU General Public License along with
-// this program; if not, write to the
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
 //
-//     Free Software Foundation, Inc.,
-//     51 Franklin Street, Fifth Floor,
-//     Boston, MA
-//     02110-1301
-//     USA
+// http://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
+
+#define L10N                                           // Localization complete.
+
 #include <iostream>
 #include <iomanip>
 #include <sstream>
 #include <vector>
 #include <algorithm>
-#include "Color.h"
-#include "text.h"
-#include "i18n.h"
+#include <main.h>
+#include <Color.h>
+#include <text.h>
+#include <i18n.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 static struct
@@ -59,25 +63,28 @@ static struct
 
 ////////////////////////////////////////////////////////////////////////////////
 Color::Color ()
-: value (0)
+: _value (0)
 {
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 Color::Color (const Color& other)
 {
-  value = other.value;
+  _value = other._value;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 Color::Color (unsigned int c)
-: value (0)
+: _value (0)
 {
-  if (!(c & _COLOR_HASFG)) value &= ~_COLOR_FG;
-  if (!(c & _COLOR_HASBG)) value &= ~_COLOR_BG;
+#ifdef FEATURE_COLOR
+  if (!(c & _COLOR_HASFG)) _value &= ~_COLOR_FG;
+  if (!(c & _COLOR_HASBG)) _value &= ~_COLOR_BG;
 
-  value = c & (_COLOR_256 | _COLOR_HASBG | _COLOR_HASFG |_COLOR_UNDERLINE |
-               _COLOR_BOLD | _COLOR_BRIGHT | _COLOR_BG | _COLOR_FG);
+  _value = c & (_COLOR_256 | _COLOR_HASBG | _COLOR_HASFG |_COLOR_UNDERLINE |
+                _COLOR_INVERSE | _COLOR_BOLD | _COLOR_BRIGHT | _COLOR_BG |
+                _COLOR_FG);
+#endif
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -93,8 +100,9 @@ Color::Color (unsigned int c)
 //   colorN 0 <= N <= 255      fg 38;5;N                    bg 48;5;N
 //   rgbRGB 0 <= R,G,B <= 5    fg 38;5;16 + R*36 + G*6 + B  bg 48;5;16 + R*36 + G*6 + B
 Color::Color (const std::string& spec)
-: value (0)
+: _value (0)
 {
+#ifdef FEATURE_COLOR
   // By converting underscores to spaces, we inherently support the old "on_red"
   // style of specifying background colors.  We consider underscores to be
   // deprecated.
@@ -122,6 +130,7 @@ Color::Color (const std::string& spec)
          if (word == "bold")      fg_value |= _COLOR_BOLD;
     else if (word == "bright")    bg_value |= _COLOR_BRIGHT;
     else if (word == "underline") fg_value |= _COLOR_UNDERLINE;
+    else if (word == "inverse")   fg_value |= _COLOR_INVERSE;
     else if (word == "on")        bg = true;
 
     // X where X is one of black, red, blue ...
@@ -148,7 +157,7 @@ Color::Color (const std::string& spec)
     {
       index = atoi (word.substr (4).c_str ());
       if (index < 0 || index > 23)
-        throw std::string ("The color '") + *it + "' is not recognized.";
+        throw format (STRING_COLOR_UNRECOGNIZED, *it);
 
       if (bg)
       {
@@ -170,7 +179,7 @@ Color::Color (const std::string& spec)
       index = atoi (word.substr (3).c_str ());
       if (word.length () != 6 ||
           index < 0 || index > 555)
-        throw std::string ("The color '") + *it + "' is not recognized.";
+        throw format (STRING_COLOR_UNRECOGNIZED, *it);
 
       int r = atoi (word.substr (3, 1).c_str ());
       int g = atoi (word.substr (4, 1).c_str ());
@@ -178,7 +187,7 @@ Color::Color (const std::string& spec)
       if (r < 0 || r > 5 ||
           g < 0 || g > 5 ||
           b < 0 || b > 5)
-        throw std::string ("The color '") + *it + "' is not recognized.";
+        throw format (STRING_COLOR_UNRECOGNIZED, *it);
 
       index = 16 + r*36 + g*6 + b;
 
@@ -201,7 +210,7 @@ Color::Color (const std::string& spec)
     {
       index = atoi (word.substr (5).c_str ());
       if (index < 0 || index > 255)
-        throw std::string ("The color '") + *it + "' is not recognized.";
+        throw format (STRING_COLOR_UNRECOGNIZED, *it);
 
       upgrade ();
 
@@ -219,61 +228,68 @@ Color::Color (const std::string& spec)
       }
     }
     else if (word != "")
-      throw std::string ("The color '") + *it + "' is not recognized.";
+      throw format (STRING_COLOR_UNRECOGNIZED, *it);
   }
 
   // Now combine the fg and bg into a single color.
-  value = fg_value;
+  _value = fg_value;
   blend (Color (bg_value));
+#endif
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 Color::Color (color_id fg)
-: value (0)
+: _value (0)
 {
+#ifdef FEATURE_COLOR
   if (fg != Color::nocolor)
   {
-    value |= _COLOR_HASFG;
-    value |= fg;
+    _value |= _COLOR_HASFG;
+    _value |= fg;
   }
+#endif
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 Color::Color (color_id fg, color_id bg)
-: value (0)
+: _value (0)
 {
+#ifdef FEATURE_COLOR
   if (bg != Color::nocolor)
   {
-    value |= _COLOR_HASBG;
-    value |= (bg << 8);
+    _value |= _COLOR_HASBG;
+    _value |= (bg << 8);
   }
 
   if (fg != Color::nocolor)
   {
-    value |= _COLOR_HASFG;
-    value |= fg;
+    _value |= _COLOR_HASFG;
+    _value |= fg;
   }
+#endif
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 Color::Color (color_id fg, color_id bg, bool underline, bool bold, bool bright)
-: value (0)
+: _value (0)
 {
-  value |= ((underline ? 1 : 0) << 18)
-        |  ((bold      ? 1 : 0) << 17)
-        |  ((bright    ? 1 : 0) << 16);
+#ifdef FEATURE_COLOR
+  _value |= ((underline ? 1 : 0) << 18)
+         |  ((bold      ? 1 : 0) << 17)
+         |  ((bright    ? 1 : 0) << 16);
 
   if (bg != Color::nocolor)
   {
-    value |= _COLOR_HASBG;
-    value |= (bg << 8);
+    _value |= _COLOR_HASBG;
+    _value |= (bg << 8);
   }
 
   if (fg != Color::nocolor)
   {
-    value |= _COLOR_HASFG;
-    value |= fg;
+    _value |= _COLOR_HASFG;
+    _value |= fg;
   }
+#endif
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -285,7 +301,7 @@ Color::~Color ()
 Color& Color::operator= (const Color& other)
 {
   if (this != &other)
-    value = other.value;
+    _value = other._value;
 
   return *this;
 }
@@ -294,23 +310,28 @@ Color& Color::operator= (const Color& other)
 Color::operator std::string () const
 {
   std::string description;
-  if (value & _COLOR_BOLD) description += "bold";
+#ifdef FEATURE_COLOR
+  if (_value & _COLOR_BOLD) description += "bold";
 
-  if (value & _COLOR_UNDERLINE)
+  if (_value & _COLOR_UNDERLINE)
     description += std::string (description.length () ? " " : "") + "underline";
 
-  if (value & _COLOR_HASFG)
+  if (_value & _COLOR_INVERSE)
+    description += std::string (description.length () ? " " : "") + "inverse";
+
+  if (_value & _COLOR_HASFG)
     description += std::string (description.length () ? " " : "") + fg ();
 
-  if (value & _COLOR_HASBG)
+  if (_value & _COLOR_HASBG)
   {
     description += std::string (description.length () ? " " : "") + "on";
 
-    if (value & _COLOR_BRIGHT)
+    if (_value & _COLOR_BRIGHT)
       description += std::string (description.length () ? " " : "") + "bright";
 
     description += " " + bg ();
   }
+#endif
 
   return description;
 }
@@ -318,7 +339,7 @@ Color::operator std::string () const
 ////////////////////////////////////////////////////////////////////////////////
 Color::operator int () const
 {
-  return (int) value;
+  return (int) _value;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -326,28 +347,30 @@ Color::operator int () const
 // other take precedence.
 void Color::blend (const Color& other)
 {
+#ifdef FEATURE_COLOR
   Color c (other);
-  value |= (c.value & _COLOR_UNDERLINE);    // Always inherit underline.
+  _value |= (c._value & _COLOR_UNDERLINE);    // Always inherit underline.
+  _value |= (c._value & _COLOR_INVERSE);      // Always inherit inverse.
 
   // 16 <-- 16.
-  if (!(value   & _COLOR_256) &&
-      !(c.value & _COLOR_256))
+  if (!(_value   & _COLOR_256) &&
+      !(c._value & _COLOR_256))
   {
-    value |= (c.value & _COLOR_BOLD);       // Inherit bold.
-    value |= (c.value & _COLOR_BRIGHT);     // Inherit bright.
+    _value |= (c._value & _COLOR_BOLD);       // Inherit bold.
+    _value |= (c._value & _COLOR_BRIGHT);     // Inherit bright.
 
-    if (c.value & _COLOR_HASFG)
+    if (c._value & _COLOR_HASFG)
     {
-      value |= _COLOR_HASFG;                // There is now a color.
-      value &= ~_COLOR_FG;                  // Remove previous color.
-      value |= (c.value & _COLOR_FG);       // Apply other color.
+      _value |= _COLOR_HASFG;                 // There is now a color.
+      _value &= ~_COLOR_FG;                   // Remove previous color.
+      _value |= (c._value & _COLOR_FG);       // Apply other color.
     }
 
-    if (c.value & _COLOR_HASBG)
+    if (c._value & _COLOR_HASBG)
     {
-      value |= _COLOR_HASBG;                // There is now a color.
-      value &= ~_COLOR_BG;                  // Remove previous color.
-      value |= (c.value & _COLOR_BG);       // Apply other color.
+      _value |= _COLOR_HASBG;                 // There is now a color.
+      _value &= ~_COLOR_BG;                   // Remove previous color.
+      _value |= (c._value & _COLOR_BG);       // Apply other color.
     }
 
     return;
@@ -355,51 +378,54 @@ void Color::blend (const Color& other)
   else
   {
     // Upgrade either color, if necessary.
-    if (!(value   & _COLOR_256)) upgrade ();
-    if (!(c.value & _COLOR_256)) c.upgrade ();
+    if (!(_value   & _COLOR_256)) upgrade ();
+    if (!(c._value & _COLOR_256)) c.upgrade ();
 
     // 256 <-- 256.
-    if (c.value & _COLOR_HASFG)
+    if (c._value & _COLOR_HASFG)
     {
-      value |= _COLOR_HASFG;                  // There is now a color.
-      value &= ~_COLOR_FG;                    // Remove previous color.
-      value |= (c.value & _COLOR_FG);         // Apply other color.
+      _value |= _COLOR_HASFG;                  // There is now a color.
+      _value &= ~_COLOR_FG;                    // Remove previous color.
+      _value |= (c._value & _COLOR_FG);        // Apply other color.
     }
 
-    if (c.value & _COLOR_HASBG)
+    if (c._value & _COLOR_HASBG)
     {
-      value |= _COLOR_HASBG;                  // There is now a color.
-      value &= ~_COLOR_BG;                    // Remove previous color.
-      value |= (c.value & _COLOR_BG);         // Apply other color.
+      _value |= _COLOR_HASBG;                  // There is now a color.
+      _value &= ~_COLOR_BG;                    // Remove previous color.
+      _value |= (c._value & _COLOR_BG);        // Apply other color.
     }
   }
+#endif
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 void Color::upgrade ()
 {
-  if (!(value & _COLOR_256))
+#ifdef FEATURE_COLOR
+  if (!(_value & _COLOR_256))
   {
-    if (value & _COLOR_HASFG)
+    if (_value & _COLOR_HASFG)
     {
-      bool bold = value & _COLOR_BOLD;
-      unsigned int fg = value & _COLOR_FG;
-      value &= ~_COLOR_FG;
-      value &= ~_COLOR_BOLD;
-      value |= (bold ? fg + 7 : fg - 1);
+      bool bold = _value & _COLOR_BOLD;
+      unsigned int fg = _value & _COLOR_FG;
+      _value &= ~_COLOR_FG;
+      _value &= ~_COLOR_BOLD;
+      _value |= (bold ? fg + 7 : fg - 1);
     }
 
-    if (value & _COLOR_HASBG)
+    if (_value & _COLOR_HASBG)
     {
-      bool bright = value & _COLOR_BRIGHT;
-      unsigned int bg = (value & _COLOR_BG) >> 8;
-      value &= ~_COLOR_BG;
-      value &= ~_COLOR_BRIGHT;
-      value |= (bright ? bg + 7 : bg - 1) << 8;
+      bool bright = _value & _COLOR_BRIGHT;
+      unsigned int bg = (_value & _COLOR_BG) >> 8;
+      _value &= ~_COLOR_BG;
+      _value &= ~_COLOR_BRIGHT;
+      _value |= (bright ? bg + 7 : bg - 1) << 8;
     }
 
-    value |= _COLOR_256;
+    _value |= _COLOR_256;
   }
+#endif
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -416,32 +442,39 @@ void Color::upgrade ()
 //   256 bg               \033[48;5;Nm
 std::string Color::colorize (const std::string& input)
 {
-  if (value == 0)
+#ifdef FEATURE_COLOR
+  if (_value == 0)
     return input;
 
   int count = 0;
   std::stringstream result;
 
   // 256 color
-  if (value & _COLOR_256)
+  if (_value & _COLOR_256)
   {
     bool needTerminator = false;
 
-    if (value & _COLOR_UNDERLINE)
+    if (_value & _COLOR_UNDERLINE)
     {
       result << "\033[4m";
       needTerminator = true;
     }
 
-    if (value & _COLOR_HASFG)
+    if (_value & _COLOR_INVERSE)
     {
-      result << "\033[38;5;" << (value & _COLOR_FG) << "m";
+      result << "\033[7m";
       needTerminator = true;
     }
 
-    if (value & _COLOR_HASBG)
+    if (_value & _COLOR_HASFG)
     {
-      result << "\033[48;5;" << ((value & _COLOR_BG) >> 8) << "m";
+      result << "\033[38;5;" << (_value & _COLOR_FG) << "m";
+      needTerminator = true;
+    }
+
+    if (_value & _COLOR_HASBG)
+    {
+      result << "\033[48;5;" << ((_value & _COLOR_BG) >> 8) << "m";
       needTerminator = true;
     }
 
@@ -453,52 +486,93 @@ std::string Color::colorize (const std::string& input)
   }
 
   // 16 color
-  if (value != 0)
+  if (_value != 0)
   {
     result << "\033[";
 
-    if (value & _COLOR_BOLD)
+    if (_value & _COLOR_BOLD)
     {
       if (count++) result << ";";
       result << "1";
     }
 
-    if (value & _COLOR_UNDERLINE)
+    if (_value & _COLOR_UNDERLINE)
     {
       if (count++) result << ";";
       result << "4";
     }
 
-    if (value & _COLOR_HASFG)
+    if (_value & _COLOR_INVERSE)
     {
       if (count++) result << ";";
-      result << (29 + (value & _COLOR_FG));
+      result << "7";
     }
 
-    if (value & _COLOR_HASBG)
+    if (_value & _COLOR_HASFG)
     {
       if (count++) result << ";";
-      result << ((value & _COLOR_BRIGHT ? 99 : 39) + ((value & _COLOR_BG) >> 8));
+      result << (29 + (_value & _COLOR_FG));
+    }
+
+    if (_value & _COLOR_HASBG)
+    {
+      if (count++) result << ";";
+      result << ((_value & _COLOR_BRIGHT ? 99 : 39) + ((_value & _COLOR_BG) >> 8));
     }
 
     result << "m" << input << "\033[0m";
     return result.str ();
   }
+#endif
 
   return input;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// Remove color codes from a string.
+std::string Color::strip (const std::string& input)
+{
+#ifdef FEATURE_COLOR
+  int length = input.length ();
+  bool inside = false;
+  std::string output;
+  for (int i = 0; i < length; ++i)
+  {
+    if (inside)
+    {
+      if (input[i] == 'm')
+        inside = false;
+    }
+    else
+    {
+      if (input[i] == 033)
+        inside = true;
+      else
+        output += input[i];
+    }
+  }
+
+  return output;
+#else
+  return input;
+#endif
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 std::string Color::colorize (const std::string& input, const std::string& spec)
 {
+#ifdef FEATURE_COLOR
   Color c (spec);
   return c.colorize (input);
+#else
+  return input;
+#endif
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 bool Color::nontrivial ()
 {
-  return value != 0 ? true : false;
+  return _value != 0 ? true : false;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -514,14 +588,15 @@ int Color::find (const std::string& input)
 ////////////////////////////////////////////////////////////////////////////////
 std::string Color::fg () const
 {
-  int index = value & _COLOR_FG;
+#ifdef FEATURE_COLOR
+  int index = _value & _COLOR_FG;
 
-  if (value & _COLOR_256)
+  if (_value & _COLOR_256)
   {
-    if (value & _COLOR_HASFG)
+    if (_value & _COLOR_HASFG)
     {
       std::stringstream s;
-      s << "color" << (value & _COLOR_FG);
+      s << "color" << (_value & _COLOR_FG);
       return s.str ();
     }
   }
@@ -531,6 +606,7 @@ std::string Color::fg () const
       if (allColors[i].index == index)
         return allColors[i].english_name;
   }
+#endif
 
   return "";
 }
@@ -538,14 +614,15 @@ std::string Color::fg () const
 ////////////////////////////////////////////////////////////////////////////////
 std::string Color::bg () const
 {
-  int index = (value & _COLOR_BG) >> 8;
+#ifdef FEATURE_COLOR
+  int index = (_value & _COLOR_BG) >> 8;
 
-  if (value & _COLOR_256)
+  if (_value & _COLOR_256)
   {
-    if (value & _COLOR_HASBG)
+    if (_value & _COLOR_HASBG)
     {
       std::stringstream s;
-      s << "color" << ((value & _COLOR_BG) >> 8);
+      s << "color" << ((_value & _COLOR_BG) >> 8);
       return s.str ();
     }
   }
@@ -555,6 +632,7 @@ std::string Color::bg () const
       if (allColors[i].index == index)
         return allColors[i].english_name;
   }
+#endif
 
   return "";
 }
