@@ -1,31 +1,33 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
-// All rights reserved.
+// Copyright 2006-2012, Paul Beckingham, Federico Hernandez.
 //
-// This program is free software; you can redistribute it and/or modify it under
-// the terms of the GNU General Public License as published by the Free Software
-// Foundation; either version 2 of the License, or (at your option) any later
-// version.
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to deal
+// in the Software without restriction, including without limitation the rights
+// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+// copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
 //
-// This program is distributed in the hope that it will be useful, but WITHOUT
-// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
-// details.
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
 //
-// You should have received a copy of the GNU General Public License along with
-// this program; if not, write to the
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
 //
-//     Free Software Foundation, Inc.,
-//     51 Franklin Street, Fifth Floor,
-//     Boston, MA
-//     02110-1301
-//     USA
+// http://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
+
 #ifndef INCLUDED_DATE
 #define INCLUDED_DATE
+#define L10N                                           // Localization complete.
 
 #include <stdio.h>
 #include <string>
@@ -48,6 +50,7 @@ public:
   time_t toEpoch ();
   std::string toEpochString ();
   std::string toISO ();
+  double toJulian ();
   void toMDY (int&, int&, int&);
   const std::string toString (const std::string& format = "m/d/Y") const;
 
@@ -59,6 +62,7 @@ public:
   static bool valid (const std::string&, const std::string& format = "m/d/Y");
   static bool valid (const int, const int, const int, const int, const int, const int);
   static bool valid (const int, const int, const int);
+  static bool valid (const int, const int);
 
   static time_t easter (int year);
   static bool leapYear (int);
@@ -69,12 +73,14 @@ public:
   static int weekOfYear (const std::string&);
   static int dayOfWeek (const std::string&);
   static int monthOfYear (const std::string&);
+  static int length (const std::string&);
 
   int month () const;
   int day () const;
   int year () const;
   int weekOfYear (int) const;
   int dayOfWeek () const;
+  int dayOfYear () const;
   int hour () const;
   int minute () const;
   int second () const;
@@ -102,12 +108,14 @@ public:
   void operator++ ();    // Prefix
   void operator++ (int); // Postfix
 
+  static const std::vector <std::string> get_relatives ();
+
 private:
   bool isEpoch (const std::string&);
   bool isRelativeDate (const std::string&);
 
 protected:
-  time_t mT;
+  time_t _t;
 };
 
 #endif
