@@ -1,38 +1,40 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
-// All rights reserved.
+// Copyright 2006-2012, Paul Beckingham, Federico Hernandez.
 //
-// This program is free software; you can redistribute it and/or modify it under
-// the terms of the GNU General Public License as published by the Free Software
-// Foundation; either version 2 of the License, or (at your option) any later
-// version.
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to deal
+// in the Software without restriction, including without limitation the rights
+// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+// copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
 //
-// This program is distributed in the hope that it will be useful, but WITHOUT
-// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
-// details.
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
 //
-// You should have received a copy of the GNU General Public License along with
-// this program; if not, write to the
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
 //
-//     Free Software Foundation, Inc.,
-//     51 Franklin Street, Fifth Floor,
-//     Boston, MA
-//     02110-1301
-//     USA
+// http://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
+#define L10N                                           // Localization complete.
+
 #include <fstream>
 #include <glob.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <pwd.h>
 #include <unistd.h>
-#include "Path.h"
-#include "../auto.h"
+#include <Path.h>
+#include <cmake.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 Path::Path ()
@@ -43,13 +45,13 @@ Path::Path ()
 Path::Path (const Path& other)
 {
   if (this != &other)
-    data = other.data;
+    _data = other._data;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 Path::Path (const std::string& in)
 {
-  data = expand (in);
+  _data = expand (in);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -61,7 +63,7 @@ Path::~Path ()
 Path& Path::operator= (const Path& other)
 {
   if (this != &other)
-    this->data = other.data;
+    this->_data = other._data;
 
   return *this;
 }
@@ -69,36 +71,36 @@ Path& Path::operator= (const Path& other)
 ////////////////////////////////////////////////////////////////////////////////
 bool Path::operator== (const Path& other)
 {
-  return data == other.data;
+  return _data == other._data;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 Path::operator std::string () const
 {
-  return data;
+  return _data;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 std::string Path::name () const
 {
-  if (data.length ())
+  if (_data.length ())
   {
-    std::string::size_type slash = data.rfind ('/');
+    std::string::size_type slash = _data.rfind ('/');
     if (slash != std::string::npos)
-      return data.substr (slash + 1, std::string::npos);
+      return _data.substr (slash + 1, std::string::npos);
   }
 
- return data;
+ return _data;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 std::string Path::parent () const
 {
-  if (data.length ())
+  if (_data.length ())
   {
-    std::string::size_type slash = data.rfind ('/');
+    std::string::size_type slash = _data.rfind ('/');
     if (slash != std::string::npos)
-      return data.substr (0, slash);
+      return _data.substr (0, slash);
   }
 
   return "";
@@ -107,11 +109,11 @@ std::string Path::parent () const
 ////////////////////////////////////////////////////////////////////////////////
 std::string Path::extension () const
 {
-  if (data.length ())
+  if (_data.length ())
   {
-    std::string::size_type dot = data.rfind ('.');
+    std::string::size_type dot = _data.rfind ('.');
     if (dot != std::string::npos)
-      return data.substr (dot + 1, std::string::npos);
+      return _data.substr (dot + 1, std::string::npos);
   }
 
   return "";
@@ -120,14 +122,14 @@ std::string Path::extension () const
 ////////////////////////////////////////////////////////////////////////////////
 bool Path::exists () const
 {
-  return access (data.c_str (), F_OK) ? false : true;
+  return access (_data.c_str (), F_OK) ? false : true;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 bool Path::is_directory () const
 {
   struct stat s = {0};
-  if (! stat (data.c_str (), &s) &&
+  if (! stat (_data.c_str (), &s) &&
       s.st_mode & S_IFDIR)
     return true;
 
@@ -137,7 +139,7 @@ bool Path::is_directory () const
 ////////////////////////////////////////////////////////////////////////////////
 bool Path::is_absolute () const
 {
-  if (data.length () && data.substr (0, 1) == "/")
+  if (_data.length () && _data.substr (0, 1) == "/")
     return true;
 
   return false;
@@ -146,19 +148,19 @@ bool Path::is_absolute () const
 ////////////////////////////////////////////////////////////////////////////////
 bool Path::readable () const
 {
-  return access (data.c_str (), R_OK) ? false : true;
+  return access (_data.c_str (), R_OK) ? false : true;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 bool Path::writable () const
 {
-  return access (data.c_str (), W_OK) ? false : true;
+  return access (_data.c_str (), W_OK) ? false : true;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 bool Path::executable () const
 {
-  return access (data.c_str (), X_OK) ? false : true;
+  return access (_data.c_str (), X_OK) ? false : true;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
