@@ -1,51 +1,55 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
-// All rights reserved.
+// Copyright 2006-2012, Paul Beckingham, Federico Hernandez.
 //
-// This program is free software; you can redistribute it and/or modify it under
-// the terms of the GNU General Public License as published by the Free Software
-// Foundation; either version 2 of the License, or (at your option) any later
-// version.
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to deal
+// in the Software without restriction, including without limitation the rights
+// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+// copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
 //
-// This program is distributed in the hope that it will be useful, but WITHOUT
-// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
-// details.
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
 //
-// You should have received a copy of the GNU General Public License along with
-// this program; if not, write to the
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
 //
-//     Free Software Foundation, Inc.,
-//     51 Franklin Street, Fifth Floor,
-//     Boston, MA
-//     02110-1301
-//     USA
+// http://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
+#define L10N                                           // Localization complete.
+
 #include <algorithm>
 #include <iostream>
 #include <sstream>
+#include <stack>
 #include <Context.h>
 #include <text.h>
 #include <util.h>
+#include <i18n.h>
 #include <main.h>
 
 extern Context context;
 
-////////////////////////////////////////////////////////////////////////////////
-static bool followUpstream (const Task&, const Task&, std::vector <std::string>&);
-
 ////////////////////////////////////////////////////////////////////////////////
 // A task is blocked if it depends on tasks that are pending or waiting.
+//
+//   1 --> 2(pending)    = blocked
+//   3 --> 4(completed)  = not blocked any more
 bool dependencyIsBlocked (const Task& task)
 {
-  if (task.has ("depends"))
+  std::string depends = task.get ("depends");
+  if (depends != "")
   {
-    std::string depends = task.get ("depends");
-    const std::vector <Task>& all = context.tdb.getAllPending ();
+    const std::vector <Task>& all = context.tdb2.pending.get_tasks ();
     std::vector <Task>::const_iterator it;
     for (it = all.begin (); it != all.end (); ++it)
       if ((it->getStatus () == Task::pending  ||
@@ -62,7 +66,7 @@ void dependencyGetBlocked (const Task& task, std::vector <Task>& blocked)
 {
   std::string uuid = task.get ("uuid");
 
-  const std::vector <Task>& all = context.tdb.getAllPending ();
+  const std::vector <Task>& all = context.tdb2.pending.get_tasks ();
   std::vector <Task>::const_iterator it;
   for (it = all.begin (); it != all.end (); ++it)
     if ((it->getStatus () == Task::pending  ||
@@ -80,7 +84,7 @@ bool dependencyIsBlocking (const Task& task)
 {
   std::string uuid = task.get ("uuid");
 
-  const std::vector <Task>& all = context.tdb.getAllPending ();
+  const std::vector <Task>& all = context.tdb2.pending.get_tasks ();
   std::vector <Task>::const_iterator it;
   for (it = all.begin (); it != all.end (); ++it)
     if ((it->getStatus () == Task::pending  ||
@@ -98,7 +102,7 @@ void dependencyGetBlocking (const Task& task, std::vector <Task>& blocking)
   std::string depends = task.get ("depends");
   if (depends != "")
   {
-    const std::vector <Task>& all = context.tdb.getAllPending ();
+    const std::vector <Task>& all = context.tdb2.pending.get_tasks ();
     std::vector <Task>::const_iterator it;
     for (it = all.begin (); it != all.end (); ++it)
       if ((it->getStatus () == Task::pending  ||
@@ -109,48 +113,38 @@ void dependencyGetBlocking (const Task& task, std::vector <Task>& blocking)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// Terminology:
-//   -->    if a depends on b, then it can be said that a --> b
-//   Head   if a --> b, then b is the head
-//   Tail   if a --> b, then a is the tail
-//
-// Algorithm:
-//   Keep walking the chain, recording the links (a --> b, b --> c, ...) until
-//   either the end of the chain is found (therefore not circular), or the chain
-//   loops and a repeat link is spotted (therefore circular).
+// Returns true if the supplied task adds a cycle to the dependency chain.
 bool dependencyIsCircular (const Task& task)
 {
-  std::vector <std::string> seen;
-  return followUpstream (task, task, seen);
-}
+  std::stack <Task> s;
+  std::vector <std::string> deps_current;
 
-////////////////////////////////////////////////////////////////////////////////
-// Returns true if a task is encountered twice in a chain walk, and therefore
-// indicates circularity.  Recursive.
-static bool followUpstream (
-  const Task& task,
-  const Task& original,
-  std::vector <std::string>& seen)
-{
-  std::vector <Task> blocking;
-  dependencyGetBlocking (task, blocking);
-  foreach (b, blocking)
+  std::string task_uuid = task.get ("uuid");
+
+  s.push (task);
+  while (!s.empty ())
   {
-    std::string link = task.get ("uuid") + " -> " + b->get ("uuid");
+    Task& current = s.top ();
+    current.getDependencies (deps_current);
 
-    // Have we seen this link before?  If so, circularity has been detected.
-    if (std::find (seen.begin (), seen.end (), link) != seen.end ())
-      return true;
+    // This is a basic depth first search that always terminates given the
+    // assumption that any cycles in the dependency graph must have been
+    // introduced by the task that is being checked.
+    // Since any previous cycles would have been prevented by this very
+    // function, this is a reasonable assumption.
+    for (unsigned int i = 0; i < deps_current.size (); i++)
+    {
+      context.tdb2.get (deps_current[i], current);
 
-    seen.push_back (link);
+      if (task_uuid == current.get ("uuid"))
+        {
+          // Cycle found, initial task reached for the second time!
+          return true;
+        }
 
-    // Use 'original' over '*b' if they both refer to the same task, otherwise
-    // '*b' is from TDB's committed list, and lacks recent modifications.
-    if (followUpstream (
-          (b->get ("uuid") == original.get ("uuid") ? original : *b),
-          original,
-          seen))
-      return true;
+      s.push (current);
+    }
+    s.pop ();
   }
 
   return false;
@@ -159,7 +153,7 @@ static bool followUpstream (
 ////////////////////////////////////////////////////////////////////////////////
 // Determine whether a dependency chain is being broken, assuming that 'task' is
 // either completed or deleted.
-//           
+//
 //   blocked task blocking action
 //   ------- ---- -------- -----------------------------
 //           [1]  2        Chain broken
@@ -199,8 +193,11 @@ void dependencyChainOnComplete (Task& task)
     // Nag about broken chain.
     if (context.config.getBoolean ("dependency.reminder"))
     {
-      std::cout << "Task " << task.id << " is blocked by:\n";
-      foreach (b, blocking)
+      std::cout << format (STRING_DEPEND_BLOCKED, task.id)
+                << "\n";
+
+      std::vector <Task>::iterator b;
+      for (b = blocking.begin (); b != blocking.end (); ++b)
         std::cout << "  " << b->id << " " << b->get ("description") << "\n";
     }
 
@@ -209,30 +206,35 @@ void dependencyChainOnComplete (Task& task)
     {
       if (context.config.getBoolean ("dependency.reminder"))
       {
-        std::cout << "and is blocking:\n";
-        foreach (b, blocked)
+        std::cout << STRING_DEPEND_BLOCKING
+                  << "\n";
+
+        std::vector <Task>::iterator b;
+        for (b = blocked.begin (); b != blocked.end (); ++b)
           std::cout << "  " << b->id << " " << b->get ("description") << "\n";
       }
 
       if (!context.config.getBoolean ("dependency.confirmation") ||
-          confirm ("Would you like the dependency chain fixed?"))
+          confirm (STRING_DEPEND_FIX_CHAIN))
       {
         // Repair the chain - everything in blocked should now depend on
         // everything in blocking, instead of task.id.
-        foreach (left, blocked)
+        std::vector <Task>::iterator left;
+        std::vector <Task>::iterator right;
+        for (left = blocked.begin (); left != blocked.end (); ++left)
         {
           left->removeDependency (task.id);
 
-          foreach (right, blocking)
+          for (right = blocking.begin (); right != blocking.end (); ++right)
             left->addDependency (right->id);
         }
 
-        // Now update TDB, now that the updates have all occurred.
-        foreach (left, blocked)
-          context.tdb.update (*left);
+        // Now update TDB2, now that the updates have all occurred.
+        for (left = blocked.begin (); left != blocked.end (); ++left)
+          context.tdb2.modify (*left);
 
-        foreach (right, blocking)
-          context.tdb.update (*right);
+        for (right = blocking.begin (); right != blocking.end (); ++right)
+          context.tdb2.modify (*right);
       }
     }
   }
@@ -250,8 +252,11 @@ void dependencyChainOnStart (Task& task)
     // broken chain.
     if (blocking.size ())
     {
-      std::cout << "Task " << task.id << " is blocked by:\n";
-      foreach (b, blocking)
+      std::cout << format (STRING_DEPEND_BLOCKED, task.id)
+                << "\n";
+
+      std::vector <Task>::iterator b;
+      for (b = blocking.begin (); b != blocking.end (); ++b)
         std::cout << "  " << b->id << " " << b->get ("description") << "\n";
     }
   }
@@ -287,13 +292,14 @@ void dependencyChainOnModify (Task& before, Task& after)
 
     // before   dep:2,3
     // after    dep:2
-    // 
+    //
     // any tasks blocked by after might should be repaired to depend on 3.
 
     std::vector <Task> blocked;
     dependencyGetBlocked (after, blocked);
 
-    foreach (b, blocked)
+    std::vector <Task>::iterator b;
+    for (b = blocked.begin (); b != blocked.end (); ++b)
     {
       std::cout << "# dependencyChainOnModify\n";
     }
