@@ -1,62 +1,113 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
-// All rights reserved.
+// Copyright 2006-2012, Paul Beckingham, Federico Hernandez.
 //
-// This program is free software; you can redistribute it and/or modify it under
-// the terms of the GNU General Public License as published by the Free Software
-// Foundation; either version 2 of the License, or (at your option) any later
-// version.
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to deal
+// in the Software without restriction, including without limitation the rights
+// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+// copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
 //
-// This program is distributed in the hope that it will be useful, but WITHOUT
-// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
-// details.
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
 //
-// You should have received a copy of the GNU General Public License along with
-// this program; if not, write to the
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
 //
-//     Free Software Foundation, Inc.,
-//     51 Franklin Street, Fifth Floor,
-//     Boston, MA
-//     02110-1301
-//     USA
+// http://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
+#define L10N                                           // Localization complete.
+
 #include <algorithm>
 #include <iostream>
 #include <sstream>
+#include <iomanip>
 #include <vector>
 #include <string>
 #include <strings.h>
 #include <ctype.h>
-#include "Context.h"
-#include "util.h"
-#include "text.h"
+#include <Context.h>
+#include <util.h>
+#include <text.h>
+#include <utf8.h>
+#include <i18n.h>
 
 extern Context context;
 
 static const char* newline = "\n";
 static const char* noline  = "";
 
+static void replace_positional (std::string&, const std::string&, const std::string&);
+
 ///////////////////////////////////////////////////////////////////////////////
 void wrapText (
   std::vector <std::string>& lines,
   const std::string& text,
-  const int width)
+  const int width,
+  bool hyphenate)
 {
   std::string copy = text;
   std::string line;
 
-  while (copy.length ())
+  int modified_width = width > 0 ? width : 1;
+
+  while (copy.length ())  // Used as Boolean, therefore UTF8 safe.
   {
-    extractLine (copy, line, width);
+    extractLine (copy, line, modified_width, hyphenate);
     lines.push_back (line);
   }
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// UTF-8
+void splitq (
+  std::vector<std::string>& results,
+  const std::string& input,
+  const char delimiter)
+{
+  results.clear ();
+
+  std::string::size_type start = 0;
+  std::string::size_type i = 0;
+  std::string word;
+  bool in_quote = false;
+  char quote;
+
+  while (utf8_next_char (input, i))
+  {
+    if (in_quote)
+    {
+      if (input[i] == quote)
+        in_quote = false;
+    }
+    else
+    {
+      if (input[i] == delimiter)
+      {
+        results.push_back (unquoteText (input.substr (start, i - start)));
+        start = i + 1;
+      }
+      else if (input[i] == '\'' ||
+               input[i] == '"')
+      {
+        quote = input[i];
+        in_quote = true;
+      }
+    }
+  }
+
+  results.push_back (unquoteText (input.substr (start)));
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 void split (
   std::vector<std::string>& results,
@@ -92,7 +143,7 @@ void split_minimal (
     start = i + 1;
   }
 
-  if (input.length ())
+  if (start < input.length ())
     results.push_back (input.substr (start));
 }
 
@@ -117,28 +168,6 @@ void split (
     results.push_back (input.substr (start));
 }
 
-////////////////////////////////////////////////////////////////////////////////
-void split_minimal (
-  std::vector<std::string>& results,
-  const std::string& input,
-  const std::string& delimiter)
-{
-  results.clear ();
-  std::string::size_type length = delimiter.length ();
-
-  std::string::size_type start = 0;
-  std::string::size_type i;
-  while ((i = input.find (delimiter, start)) != std::string::npos)
-  {
-    if (i != start)
-      results.push_back (input.substr (start, i - start));
-    start = i + length;
-  }
-
-  if (input.length ())
-    results.push_back (input.substr (start));
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 void join (
   std::string& result,
@@ -214,54 +243,149 @@ std::string unquoteText (const std::string& input)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-void extractLine (std::string& text, std::string& line, int length)
+int longestWord (const std::string& input)
 {
-  size_t eol = text.find ("\n");
+  int longest = 0;
+  int length = 0;
+  std::string::size_type i = 0;
+  int character;
 
-  // Special case: found \n in first length characters.
-  if (eol != std::string::npos && eol < (unsigned) length)
+  while ((character = utf8_next_char (input, i)))
   {
-    line = text.substr (0, eol); // strip \n
-    text = text.substr (eol + 1);
-    return;
+    if (character == ' ')
+    {
+      if (length > longest)
+        longest = length;
+
+      length = 0;
+    }
+    else
+      ++length;
   }
 
-  // Special case: no \n, and less than length characters total.
-  // special case: text.find ("\n") == std::string::npos && text.length () < length
-  if (eol == std::string::npos && characters (text) <= length)
+  return longest;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+int longestLine (const std::string& input)
+{
+  int longest = 0;
+  int length = 0;
+  std::string::size_type i = 0;
+  int character;
+
+  while ((character = utf8_next_char (input, i)))
+  {
+    if (character == '\n')
+    {
+      if (length > longest)
+        longest = length;
+
+      length = 0;
+    }
+    else
+      ++length;
+  }
+
+  if (length > longest)
+    longest = length;
+
+  return longest;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// Walk the input text looking for a break point.  A break point is one of:
+//   - EOS
+//   - \n
+//   - last space before 'length' characters
+//   - first 'length' characters
+void extractLine (
+  std::string& text,
+  std::string& line,
+  int length,
+  bool hyphenate)
+{
+  std::string::size_type bytes      = 0;
+  std::string::size_type previous   = std::string::npos;
+  std::string::size_type last_space = std::string::npos;
+  int character;
+  int chars;
+  for (chars = 0; chars < length; ++chars)
+  {
+    previous = bytes;
+    character = utf8_next_char (text, bytes);
+
+    // Record last seen space.
+    if (character == ' ')
+      last_space = previous;
+
+    // Newline is an early break point.
+    if (character == '\n')
+    {
+      line = text.substr (0, bytes - 1);
+      text = text.substr (bytes);
+      return;
+    }
+
+    // EOS is an early break point.
+    if (character == 0)
+    {
+      line = text;
+      text = "";
+      return;
+    }
+  }
+
+  // Case where EOS was not quite reached.
+  //  012345
+  // |eleven|\0
+  if (text[bytes] == '\0')
   {
     line = text;
     text = "";
     return;
   }
 
-  // Safe to ASSERT text.length () > length
-
-  // Look for the last space prior to length
-  eol = length;
-  while (eol && text[eol] != ' ' && text[eol] != '\n')
-    --eol;
-
-  // If a space was found, break there.
-  if (eol)
+  // Case where a word ends at the right margin.
+  //  012345
+  // |eleven|_
+  if (text[bytes] == ' ')
   {
-    line = text.substr (0, eol);
-    text = text.substr (eol + 1);
+    line = text.substr (0, bytes);
+    text = text.substr (bytes + 1);
+    return;
   }
 
-  // If no space was found, hyphenate.
+  // Case where a word straddles the margin, but there is an earlier space
+  // to break on.
+  //  012345
+  // |one_tw|o
+  if (last_space != std::string::npos)
+  {
+    line = text.substr (0, last_space);
+    text = text.substr (last_space + 1);
+    return;
+  }
+
+  // Case where a word needs to be split, and there is no last_space.
+  // Hyphenation becomes the issue.
+  //  012345
+  // |fiftee|n
+  // |fifte-|en
   else
   {
-    if (length > 1)
+    if (hyphenate)
     {
-      line = text.substr (0, length - 1) + "-";
-      text = text.substr (length - 1);
+      line = text.substr (0, previous) + "-";
+      text = text.substr (previous);
     }
     else
     {
-      line = text.substr (0, 1);
-      text = text.substr (length);
+      line = text.substr (0, bytes);
+      text = text.substr (bytes);
     }
+
+    return;
   }
 }
 
@@ -344,10 +468,7 @@ std::string commify (const std::string& data)
 std::string lowerCase (const std::string& input)
 {
   std::string output = input;
-  for (int i = 0; i < (int) input.length (); ++i)
-    if (isupper (input[i]))
-      output[i] = tolower (input[i]);
-
+  std::transform (output.begin (), output.end (), output.begin (), tolower);
   return output;
 }
 
@@ -355,10 +476,7 @@ std::string lowerCase (const std::string& input)
 std::string upperCase (const std::string& input)
 {
   std::string output = input;
-  for (int i = 0; i < (int) input.length (); ++i)
-    if (islower (input[i]))
-      output[i] = toupper (input[i]);
-
+  std::transform (output.begin (), output.end (), output.begin (), toupper);
   return output;
 }
 
@@ -373,13 +491,26 @@ std::string ucFirst (const std::string& input)
   return output;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+const std::string str_replace (
+  std::string &str,
+  const std::string& search,
+  const std::string& replacement)
+{
+  std::string::size_type pos = 0;
+  while ((pos = str.find (search, pos)) != std::string::npos)
+  {
+    str.replace (pos, search.length (), replacement);
+    pos += replacement.length ();
+  }
+
+  return str;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 const char* optionalBlankLine ()
 {
-  if (context.config.getBoolean ("blanklines") == true) // no i18n
-    return newline;
-
-  return noline;
+  return context.verbose ("blank") ? newline : noline;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -389,7 +520,8 @@ void guess (
   std::string& candidate)
 {
   std::vector <std::string> matches;
-  autoComplete (candidate, options, matches);
+  autoComplete (candidate, options, matches,
+                context.config.getInteger ("abbreviation.minimum"));
   if (1 == matches.size ())
     candidate = matches[0];
 
@@ -400,11 +532,7 @@ void guess (
   {
     std::sort (matches.begin (), matches.end ());
 
-    std::string error = "Ambiguous "; // TODO i18n
-    error += type;
-    error += " '";
-    error += candidate;
-    error += "' - could be either of "; // TODO i18n
+    std::string error = format (STRING_TEXT_AMBIGUOUS, type, candidate);
     for (size_t i = 0; i < matches.size (); ++i)
     {
       if (i)
@@ -416,6 +544,16 @@ void guess (
   }
 }
 
+////////////////////////////////////////////////////////////////////////////////
+bool nontrivial (const std::string& input)
+{
+  for (size_t i = 0; i < input.length (); ++i)
+    if (!isspace (input[i]))
+      return true;
+
+  return false;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 bool digitsOnly (const std::string& input)
 {
@@ -472,7 +610,7 @@ bool isWordStart (const std::string& input, std::string::size_type pos)
 // Result for pos: ....y......y
 bool isWordEnd (const std::string& input, std::string::size_type pos)
 {
-  // Short circuit: no input means no word start.
+  // Short circuit: no input means no word end.
   if (input.length () == 0)
     return false;
 
@@ -489,6 +627,25 @@ bool isWordEnd (const std::string& input, std::string::size_type pos)
   return false;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+//          Input: hello, world
+// Result for pos: ....y......y
+//
+//          Input: (one) two
+// Result for pos: y..yy...y
+bool isTokenEnd (const std::string& input, std::string::size_type pos)
+{
+  // Delegate.
+  if (isWordEnd (input, pos))
+    return true;
+
+  // Punctuation divides tokens.
+  if (pos < input.length () && isPunctuation (input[pos]))
+    return true;
+
+  return false;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Override of ispunct, that considers #, $ and @ not to be punctuation.
 //
@@ -518,6 +675,18 @@ bool compare (
   return left == right;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+bool closeEnough (const std::string& reference, const std::string& attempt)
+{
+  if (compare (reference, attempt, false))
+    return true;
+
+  if (attempt.length () < reference.length ())
+    return compare (reference.substr (0, attempt.length ()), attempt, false);
+
+  return false;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 std::string::size_type find (
   const std::string& text,
@@ -635,26 +804,7 @@ int strippedLength (const std::string& input)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// UTF8
-int characters (const std::string& str)
-{
-  int byteLength = str.length ();
-  int charLength = byteLength;
-  const char* data = str.data ();
-
-  // decrement the number of bytes for each byte that matches 0b10??????
-  // this way only the first byte of any utf8 sequence is counted
-  for (int i = 0; i < byteLength; i++)
-  {
-    // extract the two MSB and check whether they are 10
-    if ((data[i] & 0xC0) == 0x80)
-      charLength--;
-  }
-
-  return charLength;
-}
-
-////////////////////////////////////////////////////////////////////////////////
+// Truncates a long line, and include a two-character ellipsis.
 std::string cutOff (const std::string& str, std::string::size_type len)
 {
   if (str.length () > len)
@@ -670,7 +820,7 @@ std::string cutOff (const std::string& str, std::string::size_type len)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-std::string format (char value)
+const std::string format (char value)
 {
   std::stringstream s;
   s << value;
@@ -678,7 +828,7 @@ std::string format (char value)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-std::string format (int value)
+const std::string format (int value)
 {
   std::stringstream s;
   s << value;
@@ -686,7 +836,7 @@ std::string format (int value)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-std::string formatHex (int value)
+const std::string formatHex (int value)
 {
   std::stringstream s;
   s.setf (std::ios::hex, std::ios::basefield);
@@ -695,7 +845,7 @@ std::string formatHex (int value)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-std::string format (float value, int width, int precision)
+const std::string format (float value, int width, int precision)
 {
   std::stringstream s;
   s.width (width);
@@ -705,7 +855,7 @@ std::string format (float value, int width, int precision)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-std::string format (double value, int width, int precision)
+const std::string format (double value, int width, int precision)
 {
   std::stringstream s;
   s.width (width);
@@ -715,3 +865,168 @@ std::string format (double value, int width, int precision)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+const std::string format (double value)
+{
+  std::stringstream s;
+  s << std::fixed << value;
+  return s.str ();
+}
+
+////////////////////////////////////////////////////////////////////////////////
+static void replace_positional (
+  std::string& fmt,
+  const std::string& from,
+  const std::string& to)
+{
+  std::string::size_type pos = 0;
+  while ((pos = fmt.find (from, pos)) != std::string::npos)
+  {
+    fmt.replace (pos, from.length (), to);
+    pos += to.length ();
+  }
+}
+
+////////////////////////////////////////////////////////////////////////////////
+const std::string format (
+  const std::string& fmt,
+  const std::string& arg1)
+{
+  std::string output = fmt;
+  replace_positional (output, "{1}", arg1);
+  return output;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+const std::string format (
+  const std::string& fmt,
+  int arg1)
+{
+  std::string output = fmt;
+  replace_positional (output, "{1}", format (arg1));
+  return output;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+const std::string format (
+  const std::string& fmt,
+  const std::string& arg1,
+  const std::string& arg2)
+{
+  std::string output = fmt;
+  replace_positional (output, "{1}", arg1);
+  replace_positional (output, "{2}", arg2);
+  return output;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+const std::string format (
+  const std::string& fmt,
+  const std::string& arg1,
+  int arg2)
+{
+  std::string output = fmt;
+  replace_positional (output, "{1}", arg1);
+  replace_positional (output, "{2}", format (arg2));
+  return output;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+const std::string format (
+  const std::string& fmt,
+  const std::string& arg1,
+  double arg2)
+{
+  std::string output = fmt;
+  replace_positional (output, "{1}", arg1);
+  replace_positional (output, "{2}", trim (format (arg2, 6, 3)));
+  return output;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+const std::string format (
+  const std::string& fmt,
+  int arg1,
+  const std::string& arg2)
+{
+  std::string output = fmt;
+  replace_positional (output, "{1}", format (arg1));
+  replace_positional (output, "{2}", arg2);
+  return output;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+const std::string format (
+  const std::string& fmt,
+  int arg1,
+  int arg2)
+{
+  std::string output = fmt;
+  replace_positional (output, "{1}", format (arg1));
+  replace_positional (output, "{2}", format (arg2));
+  return output;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+const std::string format (
+  const std::string& fmt,
+  int arg1,
+  double arg2)
+{
+  std::string output = fmt;
+  replace_positional (output, "{1}", format (arg1));
+  replace_positional (output, "{2}", trim (format (arg2, 6, 3)));
+  return output;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+const std::string format (
+  const std::string& fmt,
+  const std::string& arg1,
+  const std::string& arg2,
+  const std::string& arg3)
+{
+  std::string output = fmt;
+  replace_positional (output, "{1}", arg1);
+  replace_positional (output, "{2}", arg2);
+  replace_positional (output, "{3}", arg3);
+  return output;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+std::string leftJustify (const int input, const int width)
+{
+  std::stringstream s;
+  s << input;
+  std::string output = s.str ();
+  return output + std::string (width - output.length (), ' ');
+}
+
+////////////////////////////////////////////////////////////////////////////////
+std::string leftJustify (const std::string& input, const int width)
+{
+  return input + std::string (width - utf8_text_length (input), ' ');
+}
+
+////////////////////////////////////////////////////////////////////////////////
+std::string rightJustifyZero (const int input, const int width)
+{
+  std::stringstream s;
+  s << std::setw (width) << std::setfill ('0') << input;
+  return s.str ();
+}
+
+////////////////////////////////////////////////////////////////////////////////
+std::string rightJustify (const int input, const int width)
+{
+  std::stringstream s;
+  s << std::setw (width) << std::setfill (' ') << input;
+  return s.str ();
+}
+
+////////////////////////////////////////////////////////////////////////////////
+std::string rightJustify (const std::string& input, const int width)
+{
+  return std::string (width - utf8_text_length (input), ' ') + input;
+}
+
+////////////////////////////////////////////////////////////////////////////////
